window.Gleam.initSearch([{"doc":"chomp","title":"chomp","content":"# chomp\n\nA lexer and parser combinator library inspired by [`elm/parser`](https://github.com/elm/parser).\n\n&gt; This package is a fork of [nibble](https://github.com/hayleigh-dot-dev/gleam-nibble) that does some things differently. Most notably, error dead ends are removed (instead favoring new functions for updating errors), and custom errors can be thrown. Some parsers have also been removed or modified and others added.\n&gt;\n&gt; If coming from nibble, be aware that things may not work the same!\n\n[![Package Version](https://img.shields.io/hexpm/v/chomp)](https://hex.pm/packages/chomp)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/chomp/)\n\n✨ This project is written in **pure Gleam** so you can use it with both the Erlang and JavaScript targets.\n\n## Quick start\n\nIf you just want to get a feel for what chomp can do, check out the example\nbelow.\n\n```gleam\nimport gleam/option.{None, Some}\nimport chomp.{do, return}\nimport chomp/lexer\n\ntype Point {\n  Point(x: Int, y: Int)\n}\n\ntype Token {\n  Num(Int)\n  LParen\n  RParen\n  Comma\n}\n\npub fn main() {\n  // Your lexer knows how to take an input string and\n  // turn it into a flat list of tokens. You define the\n  // type of token you want to use, but chomp will wrap\n  // that up in its own `Token` type that includes the\n  // source span and original lexeme for each token.\n  let lexer =\n    lexer.simple([\n      lexer.int(Num),\n      lexer.token(&quot;(&quot;, LParen),\n      lexer.token(&quot;)&quot;, RParen),\n      lexer.token(&quot;,&quot;, Comma),\n      // Skip over whitespace, we don&#39;t care about it!\n      lexer.whitespace(Nil)\n        |&gt; lexer.ignore,\n    ])\n\n  // Your parser(s!) know how to transform a list of\n  // tokens into whatever you want. You have the full\n  // power of Gleam here, so you can go wild!\n  let int_parser =\n    // Use `take_map` to only consume certain kinds of tokens and transform the\n    // result.\n    chomp.take_map(fn(tok) {\n      case tok {\n        Num(n) -&gt; Some(n)\n        _ -&gt; None\n      }\n    })\n    |&gt; chomp.or_error(&quot;expected a number&quot;)\n\n  let parser = {\n    use _ &lt;- do(chomp.token(LParen))\n    use x &lt;- do(int_parser)\n    use _ &lt;- do(chomp.token(Comma))\n    use y &lt;- do(int_parser)\n    use _ &lt;- do(chomp.token(RParen))\n\n    return(Point(x, y))\n  }\n\n  let assert Ok(tokens) = lexer.run(&quot;(1, 2)&quot;, lexer)\n  let assert Ok(point) = chomp.run(tokens, parser)\n\n  point.x //=&gt; 1\n  point.y //=&gt; 2\n}\n\n```\n\n## Examples\n\nChomp does not currently have a tutorial or guide, so the best way to learn is through the docs and [examples](./examples/). Feel free to submit a PR if you have a short, well-written parser that you think would serve as a good example!\n\n## Installation\n\nThis package can be added to your Gleam project:\n\n```sh\ngleam add chomp\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/chomp&gt;.\n","url":"index.html"},{"doc":"chomp/span","title":"Span","content":"pub type Span {\n  Span(\n    row_start: Int,\n    col_start: Int,\n    row_end: Int,\n    col_end: Int,\n  )\n}\n A source span is a range into the source string that represents the start and\n end of a lexeme in a human-readable way. That means instead of a straight index\n into the string you get a row and column for the start and end instead!\n\nSpan(row_start: Int, col_start: Int, row_end: Int, col_end: Int)\n\n\nSynonyms:\nchomp/span.Span\nchomp/span Span","url":"chomp/span.html#Span"},{"doc":"chomp/span","title":"combine","content":"pub fn combine(first: Span, second: Span) -&gt; Span\n Combine two `Span`s into one that represents the combined range.\n\n txt\n    first blah second\n 1. ~~~~~   2. ~~~~~~\n 3. ~~~~~~~~~~~~~~~~~\n \n\n 1. `Span(1, 1, 1, 6)`.\n 2. `Span(1, 12, 1, 18)`.\n 3. (combined) `Span(1, 1, 1, 18)`.\n\n Often when parsing you&#39;ll collect a start and end span that you want to combine\n and include in the AST. That&#39;s where this function is most useful!\n\n \n use start_span &lt;- do(chomp.token(LParen))\n // ...\n use end_span &lt;- do(chomp.token(RParen))\n let location = span.combine(start_span, end_span)\n\n return(SomeASTNode(.., location))\n \n\n 🚨 Note that the second `Span` must come after the first location-wise.\n\nSynonyms:\nchomp/span.combine\nchomp/span combine","url":"chomp/span.html#combine"},{"doc":"chomp/span","title":"chomp/span","content":" Spans are so useful in lexing, parsing, and error reporting that Chomp has\n a separate module dedicated to them. The nice thing about Spans is you don&#39;t\n have to include them in your parse result right away. If you find that you\n could use some helpful location information, simply take advantage of the\n fact that [`chomp.token`](../chomp.html#token) returns the location of the\n token it consumes, and [`chomp.get_pos`](../chomp.html#get_pos) allows you\n to attain the location of anything else.\n","url":"chomp/span.html"},{"doc":"chomp/lexer","title":"Error","content":"pub type Error {\n  NoMatchFound(row: Int, col: Int, lexeme: String)\n}\n\n\nNoMatchFound(row: Int, col: Int, lexeme: String)\n\n\nSynonyms:\nchomp/lexer.Error\nchomp/lexer Error","url":"chomp/lexer.html#Error"},{"doc":"chomp/lexer","title":"Lexer","content":"pub opaque type Lexer(a, mode)\n\n\n\nSynonyms:\nchomp/lexer.Lexer\nchomp/lexer Lexer","url":"chomp/lexer.html#Lexer"},{"doc":"chomp/lexer","title":"Match","content":"pub type Match(a, mode) {\n  Keep(a, mode)\n  Skip(mode)\n  Drop(mode)\n  NoMatch\n}\n When writing a custom matcher, a `Match` is what you return to tell the lexer\n what to do next.\n\nKeep(a, mode)\n Consume the accumulated input and produce a token with the given value. A\n `Keep` match can also transition the lexer into a new mode.\n\nSkip(mode)\n Skip running any additional matchers this iteration, add the next grapheme\n to the accumulated input, and run the next iteration. A `Skip` match can\n also transition the lexer into a new mode.\n\nDrop(mode)\n Drop the accumulated input and move on to the next iteration. A `Drop`\n match can also transition the lexer into a new mode. This match is useful\n for discarding input like whitespace or comments.\n\nNoMatch\n The matcher did not match the input, so the lexer should try the next\n matcher in the list (or fail if there are no more matchers).\n\nSynonyms:\nchomp/lexer.Match\nchomp/lexer Match","url":"chomp/lexer.html#Match"},{"doc":"chomp/lexer","title":"Matcher","content":"pub opaque type Matcher(a, mode)\n A `Matcher` is how we define the rules that match parts of the input string\n and turn them into tokens. At it&#39;s core, a `Match` is a function that takes\n three arguments:\n\n - The current mode of the lexer\n\n - Any input we&#39;ve accumulated so far\n\n - A lookahead of one grapheme\n\n With just these three arguments we can define arbitrary rules for consuming\n (or not) input and producing tokens!\n\n\nSynonyms:\nchomp/lexer.Matcher\nchomp/lexer Matcher","url":"chomp/lexer.html#Matcher"},{"doc":"chomp/lexer","title":"Token","content":"pub type Token(a) {\n  Token(span: Span, lexeme: String, value: a)\n}\n You use chomp&#39;s lexer to turn a string into a list of tokens that your parser\n will eventually consume. The `Token` type contains the lexeme that was consumed\n (aka the raw input string), the source [`Span`](./span.html#Span) of the\n consumed lexeme to locate it in the source, and whatever token value your lexer\n produces.\n\nToken(span: Span, lexeme: String, value: a)\n\n\nSynonyms:\nchomp/lexer.Token\nchomp/lexer Token","url":"chomp/lexer.html#Token"},{"doc":"chomp/lexer","title":"advanced","content":"pub fn advanced(\n  matchers: fn(a) -&gt; List(Matcher(b, a)),\n) -&gt; Lexer(b, a)\n An `advanced` lexer is one that can change what matchers it uses based on the\n current mode. This is useful for sophisticated lexers that might need to\n handle things like interpolated strings or indentation-sensitive syntax.\n\nSynonyms:\nchomp/lexer.advanced\nchomp/lexer advanced","url":"chomp/lexer.html#advanced"},{"doc":"chomp/lexer","title":"comment","content":"pub fn comment(\n  start: String,\n  to_value: fn(String) -&gt; a,\n) -&gt; Matcher(a, b)\n\nSynonyms:\nchomp/lexer.comment\nchomp/lexer comment","url":"chomp/lexer.html#comment"},{"doc":"chomp/lexer","title":"custom","content":"pub fn custom(\n  f: fn(a, String, String) -&gt; Match(b, a),\n) -&gt; Matcher(b, a)\n Create a custom [`Matcher`](#Matcher) that is flexible enough to do anything\n you want! The first parameter is a function that takes the current lexer mode,\n the current lexeme, and a one-grapheme lookahead.\n\n The function returns a [`Match`](#Match) that tells the lexer what to do next.\n\nSynonyms:\nchomp/lexer.custom\nchomp/lexer custom","url":"chomp/lexer.html#custom"},{"doc":"chomp/lexer","title":"drop","content":"pub fn drop(f: fn(String, String) -&gt; Bool) -&gt; Matcher(a, b)\n Create a custom [`Matcher`](#Matcher) that will consume the input and move\n to the next iteration without producing a token if it is `True` or return a\n `NoMatch` if it fails. The first parameter is a function that takes the\n current lexeme and the second parameter is a one-grapheme lookahead.\n\n Matchers created with this convienence function cannot change the lexer&#39;s\n mode or skip ahead to the next iteration without consuming the input.\n\nSynonyms:\nchomp/lexer.drop\nchomp/lexer drop","url":"chomp/lexer.html#drop"},{"doc":"chomp/lexer","title":"float","content":"pub fn float(to_value: fn(Float) -&gt; a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.float\nchomp/lexer float","url":"chomp/lexer.html#float"},{"doc":"chomp/lexer","title":"float_with_separator","content":"pub fn float_with_separator(\n  separator: String,\n  to_value: fn(Float) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.float_with_separator\nchomp/lexer float_with_separator","url":"chomp/lexer.html#float_with_separator"},{"doc":"chomp/lexer","title":"identifier","content":"pub fn identifier(\n  start: String,\n  inner: String,\n  reserved: Set(String),\n  to_value: fn(String) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.identifier\nchomp/lexer identifier","url":"chomp/lexer.html#identifier"},{"doc":"chomp/lexer","title":"ignore","content":"pub fn ignore(matcher: Matcher(a, b)) -&gt; Matcher(c, b)\n Take a matcher than might `Keep` anything and silently `Drop` anything it\n produces instead. This is useful for things like whitespace or comments\n where you want to consume some input but you don&#39;t want to emit a token.\n\nSynonyms:\nchomp/lexer.ignore\nchomp/lexer ignore","url":"chomp/lexer.html#ignore"},{"doc":"chomp/lexer","title":"int","content":"pub fn int(to_value: fn(Int) -&gt; a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.int\nchomp/lexer int","url":"chomp/lexer.html#int"},{"doc":"chomp/lexer","title":"int_with_separator","content":"pub fn int_with_separator(\n  separator: String,\n  to_value: fn(Int) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.int_with_separator\nchomp/lexer int_with_separator","url":"chomp/lexer.html#int_with_separator"},{"doc":"chomp/lexer","title":"into","content":"pub fn into(\n  matcher: Matcher(a, b),\n  f: fn(b) -&gt; b,\n) -&gt; Matcher(a, b)\n Take an existing matcher and transition to a new mode. This only runs if\n the matcher is successful and either `Keep`s or `Drop`s a value.\n\n\nSynonyms:\nchomp/lexer.into\nchomp/lexer into","url":"chomp/lexer.html#into"},{"doc":"chomp/lexer","title":"keep","content":"pub fn keep(\n  f: fn(String, String) -&gt; Result(a, Nil),\n) -&gt; Matcher(a, b)\n Create a custom [`Matcher`](#Matcher) that will consume the input and produce\n a token with the given value if it is `Ok` or return a `NoMatch` if it fails.\n The first parameter is a function that takes the current lexeme and the\n second parameter is a one-grapheme lookahead.\n\n Matchers created with this convienence function cannot change the lexer&#39;s\n mode or skip ahead to the next iteration without consuming the input.\n\nSynonyms:\nchomp/lexer.keep\nchomp/lexer keep","url":"chomp/lexer.html#keep"},{"doc":"chomp/lexer","title":"keyword","content":"pub fn keyword(\n  str: String,\n  breaker: String,\n  value: a,\n) -&gt; Matcher(a, b)\n Match exactly the given string only when the lookahead is matched by the given\n breaker _regex_. Keywords are exact strings like `let` but you wouldn&#39;t want\n to lex `letter` as `[Let, Var(&quot;tter&quot;)]` so the breaker is used so you can say\n what characters should trigger a match.\n\nSynonyms:\nchomp/lexer.keyword\nchomp/lexer keyword","url":"chomp/lexer.html#keyword"},{"doc":"chomp/lexer","title":"map","content":"pub fn map(\n  matcher: Matcher(a, b),\n  f: fn(a) -&gt; c,\n) -&gt; Matcher(c, b)\n Take an existing matcher and transform it by applying a function to the value\n it produces.\n\nSynonyms:\nchomp/lexer.map\nchomp/lexer map","url":"chomp/lexer.html#map"},{"doc":"chomp/lexer","title":"number","content":"pub fn number(\n  from_int: fn(Int) -&gt; a,\n  from_float: fn(Float) -&gt; a,\n) -&gt; Matcher(a, b)\n\nSynonyms:\nchomp/lexer.number\nchomp/lexer number","url":"chomp/lexer.html#number"},{"doc":"chomp/lexer","title":"number_with_separator","content":"pub fn number_with_separator(\n  separator: String,\n  from_int: fn(Int) -&gt; a,\n  from_float: fn(Float) -&gt; a,\n) -&gt; Matcher(a, b)\n\nSynonyms:\nchomp/lexer.number_with_separator\nchomp/lexer number_with_separator","url":"chomp/lexer.html#number_with_separator"},{"doc":"chomp/lexer","title":"run","content":"pub fn run(\n  source: String,\n  lexer: Lexer(a, Nil),\n) -&gt; Result(List(Token(a)), Error)\n\n\nSynonyms:\nchomp/lexer.run\nchomp/lexer run","url":"chomp/lexer.html#run"},{"doc":"chomp/lexer","title":"run_advanced","content":"pub fn run_advanced(\n  source: String,\n  mode: a,\n  lexer: Lexer(b, a),\n) -&gt; Result(List(Token(b)), Error)\n\n\nSynonyms:\nchomp/lexer.run_advanced\nchomp/lexer run_advanced","url":"chomp/lexer.html#run_advanced"},{"doc":"chomp/lexer","title":"simple","content":"pub fn simple(matchers: List(Matcher(a, Nil))) -&gt; Lexer(a, Nil)\n\n\nSynonyms:\nchomp/lexer.simple\nchomp/lexer simple","url":"chomp/lexer.html#simple"},{"doc":"chomp/lexer","title":"spaces","content":"pub fn spaces(token: a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.spaces\nchomp/lexer spaces","url":"chomp/lexer.html#spaces"},{"doc":"chomp/lexer","title":"spaces_","content":"pub fn spaces_(to_value: fn(String) -&gt; a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.spaces_\nchomp/lexer spaces_","url":"chomp/lexer.html#spaces_"},{"doc":"chomp/lexer","title":"string","content":"pub fn string(\n  char: String,\n  to_value: fn(String) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.string\nchomp/lexer string","url":"chomp/lexer.html#string"},{"doc":"chomp/lexer","title":"symbol","content":"pub fn symbol(\n  str: String,\n  breaker: String,\n  value: a,\n) -&gt; Matcher(a, b)\n Match exactly the given string only when the lookahead is matched by the given\n breaker _regex_. This is an alias of [`keyword`](#keyword) but it can be\n helpful to separate the two concepts.\n\nSynonyms:\nchomp/lexer.symbol\nchomp/lexer symbol","url":"chomp/lexer.html#symbol"},{"doc":"chomp/lexer","title":"then","content":"pub fn then(\n  matcher: Matcher(a, b),\n  f: fn(a) -&gt; Match(c, b),\n) -&gt; Matcher(c, b)\n Take an existing matcher and transform it by applying a function to the value\n it producs. The function you provide can return a different [`Match`](#Match)\n so you can, for example, take a matcher that `Keep`s a value and turn it into\n a matcher that `Drop`s the value instead. This is out [`ignore`](#ignore) works!\n\nSynonyms:\nchomp/lexer.then\nchomp/lexer then","url":"chomp/lexer.html#then"},{"doc":"chomp/lexer","title":"token","content":"pub fn token(str: String, value: a) -&gt; Matcher(a, b)\n Match exactly the given string with no lookahead and produce the given value.\n\nSynonyms:\nchomp/lexer.token\nchomp/lexer token","url":"chomp/lexer.html#token"},{"doc":"chomp/lexer","title":"try_identifier","content":"pub fn try_identifier(\n  start: String,\n  inner: String,\n  reserved: Set(String),\n  to_value: fn(String) -&gt; a,\n) -&gt; Result(Matcher(a, b), CompileError)\n\n\nSynonyms:\nchomp/lexer.try_identifier\nchomp/lexer try_identifier","url":"chomp/lexer.html#try_identifier"},{"doc":"chomp/lexer","title":"variable","content":"pub fn variable(\n  reserved: Set(String),\n  to_value: fn(String) -&gt; a,\n) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.variable\nchomp/lexer variable","url":"chomp/lexer.html#variable"},{"doc":"chomp/lexer","title":"whitespace","content":"pub fn whitespace(token: a) -&gt; Matcher(a, b)\n\n\nSynonyms:\nchomp/lexer.whitespace\nchomp/lexer whitespace","url":"chomp/lexer.html#whitespace"},{"doc":"chomp/lexer","title":"chomp/lexer","content":" Chomp takes a different approach to many other parser combinator libraries\n by also providing a _lexer_ combinator module that you use to turn an input\n string into a list of tokens.\n\n Parser combinators are a powerful and flexible way to build parsers, but\n they offer come at a performance cost compared to hand-written parsers or\n parser generators. On the other hand, writing a lexer by hand can be a bit\n tedious and difficult.\n\n Chomp aims to provide a happy middle-ground by making it easy to produce\n OK-performing lexers and then use parser combinators that can be much faster\n working on the smaller token stream.\n","url":"chomp/lexer.html"},{"doc":"chomp","title":"Error","content":"pub type Error(e, tok) {\n  Custom(e)\n  EndOfInput\n  Expected(tok, got: tok)\n  Unexpected(tok)\n  BadParser(String)\n}\n The `Error` type represents all of the ways that a parser can fail. It has\n two type parameters, `e` and `tok`. See the [`Parser`](#Parser) type for more\n information about them.\n\nCustom(e)\n A custom error.\n\nEndOfInput\n There are no more tokens to consume, but the parser required some.\n\nExpected(tok, got: tok)\n The parser expected a certain token but got a different one instead.\n\nUnexpected(tok)\n The parser encountered an unexpected token. This error is not very specific,\n so it&#39;s often best to replace it using the [`or_error`](#or_error) function\n when possible.\n\nBadParser(String)\n A parser was called with incorrect input.\n\nSynonyms:\nchomp.Error\nchomp Error","url":"chomp.html#Error"},{"doc":"chomp","title":"Loop","content":"pub type Loop(a, state) {\n  Continue(state)\n  Break(a)\n}\n This type is very similar to\n [`list.ContinueOrStop`](https://hexdocs.pm/gleam_stdlib/gleam/list.html#ContinueOrStop).\n\nContinue(state)\n Continue parsing with the new state.\n\nBreak(a)\n Stop parsing and return a result.\n\nSynonyms:\nchomp.Loop\nchomp Loop","url":"chomp.html#Loop"},{"doc":"chomp","title":"Parser","content":"pub opaque type Parser(a, e, tok, ctx)\n The `Parser` type has four parameters; let&#39;s take a look at each of them:\n\n \n Parser(a, e, tok, ctx)\n \n\n 1) `a` is the type of value that the parser knows how to produce. If you were\n   writing a parser for a programming language, this might be your expression\n   type.\n\n 2) `e` is the type of custom error that you can choose to throw. This could\n   be a `String` for simple errors such as &quot;I expected an expression&quot; or\n   something more complex.\n\n 3) `tok` is the type of tokens that the parser knows how to consume. You can\n   take a look at the [`Token`](./chomp/lexer#Token) type for a bit more info,\n   but note that it&#39;s not necessary for the token stream to come from chomp&#39;s\n   lexer.\n\n 4) `ctx` is used to make error reporting nicer. You can place a parser into a\n   custom context. When the parser runs the context gets pushed into a stack.\n   If the parser fails you can see the context stack in the error message,\n   which can make error reporting and debugging much easier! See the [`in`](#in)\n   function for more details.\n\n It can get a bit repetitive to write out the `Parser` type with all of the\n parameters in type annotations, so it&#39;s common to make a type alias for it in\n your parser.\n\n \n import chomp\n\n type Parser(a) = chomp.Parser(a, MyErrorType, TokenType, Context)\n\n // now your parsers can use the alias like so:\n fn expression() -&gt; Parser(Expression) {\n   ...\n }\n \n\n\nSynonyms:\nchomp.Parser\nchomp Parser","url":"chomp.html#Parser"},{"doc":"chomp","title":"any","content":"pub fn any() -&gt; Parser(a, b, a, c)\n Parse a single token of any type and return it.\n\nSynonyms:\nchomp.any\nchomp any","url":"chomp.html#any"},{"doc":"chomp","title":"backtrackable","content":"pub fn backtrackable(\n  parser: Parser(a, b, c, d),\n) -&gt; Parser(a, b, c, d)\n By default, parsers will not backtrack if they fail after consuming at least\n one token. Passing a parser to `backtrackable` will change this behaviour and\n allows us to jump back to the state of the parser before it consumed any input\n and try another one.\n\n This is most useful when you want to quickly try a few different parsers using\n [`one_of`](#one_of).\n\n 🚨 Backtracking parsers can drastically reduce performance, so you should avoid\n them where possible. A common reason folks reach for backtracking is when they\n want to try multiple branches that start with the same token or same sequence\n of tokens.\n\n To avoid backtracking in these cases, you can create an intermediate parser\n that consumes the common tokens _and then_ use [`one_of`](#one_of) to try\n the different branches.\n\nSynonyms:\nchomp.backtrackable\nchomp backtrackable","url":"chomp.html#backtrackable"},{"doc":"chomp","title":"do","content":"pub fn do(\n  parser: Parser(a, b, c, d),\n  f: fn(a) -&gt; Parser(e, b, c, d),\n) -&gt; Parser(e, b, c, d)\n Do the first `parser`, then apply `f` to its result, which returns another\n parser and is subsequently run. If the first parser fails, `f` is not called.\n Because this parser is so common, we normally import it unqualified.\n\n This is very useful for running parsers in sequence with Gleam&#39;s `use` syntax.\n\n \n import chomp.{do, return}\n\n fn point() {\n   use _ &lt;- do(chomp.token(LParen))\n   use x &lt;- do(int_parser())\n   use _ &lt;- do(chomp.token(Comma))\n   use y &lt;- do(int_parser())\n   use _ &lt;- do(chomp.token(RParen))\n\n   return(Point(x, y))\n }\n \n\n (See the [main page](https://hexdocs.pm/chomp) for a complete example)\n\nSynonyms:\nchomp.do\nchomp do","url":"chomp.html#do"},{"doc":"chomp","title":"do_in","content":"pub fn do_in(\n  context: a,\n  parser: Parser(b, c, d, a),\n  f: fn(b) -&gt; Parser(e, c, d, a),\n) -&gt; Parser(e, c, d, a)\n A combination of [`do`](#do) and [`in`](#in).\n\nSynonyms:\nchomp.do_in\nchomp do_in","url":"chomp.html#do_in"},{"doc":"chomp","title":"end","content":"pub fn end() -&gt; Parser(Nil, a, b, c)\n Parse successfully only if at the end of the token stream.\n\nSynonyms:\nchomp.end\nchomp end","url":"chomp.html#end"},{"doc":"chomp","title":"fail","content":"pub fn fail(error: a) -&gt; Parser(b, a, c, d)\n Create a parser that consumes no tokens and always fails with the given\n error.\n\nSynonyms:\nchomp.fail\nchomp fail","url":"chomp.html#fail"},{"doc":"chomp","title":"get_pos","content":"pub fn get_pos() -&gt; Parser(Span, a, b, c)\n A parser that returns the current token position.\n\nSynonyms:\nchomp.get_pos\nchomp get_pos","url":"chomp.html#get_pos"},{"doc":"chomp","title":"in","content":"pub fn in(\n  parser: Parser(a, b, c, d),\n  context: d,\n) -&gt; Parser(a, b, c, d)\n Run a parser in a certain context. This allows you to include useful\n information—*context*—in error messages. For example, instead of a message\n such as &quot;I expected a value&quot; you could say &quot;I expected a value inside of this\n list in the function `foo`&quot;. The latter is far more user-friendly!\n\n Context also holds on to the position of the parser where it was entered.\n So if you had an `InList` context entered right after a `[` token, any errors\n encountered inside the list can reference where the list started.\n\n txt\n I found a problem in this list:\n\n | the list starts here\n v\n [a, b, ]\n        ~\n        |\n I wanted a value\n \n\nSynonyms:\nchomp.in\nchomp in","url":"chomp.html#in"},{"doc":"chomp","title":"inspect","content":"pub fn inspect(\n  parser: Parser(a, b, c, d),\n  message: String,\n) -&gt; Parser(a, b, c, d)\n Run the given parser and then inspect it&#39;s state.\nSynonyms:\nchomp.inspect\nchomp inspect","url":"chomp.html#inspect"},{"doc":"chomp","title":"is_at_end","content":"pub fn is_at_end() -&gt; Parser(Bool, a, b, c)\n Returns `True` if there are no tokens left to consume.\n\nSynonyms:\nchomp.is_at_end\nchomp is_at_end","url":"chomp.html#is_at_end"},{"doc":"chomp","title":"lazy","content":"pub fn lazy(\n  parser: fn() -&gt; Parser(a, b, c, d),\n) -&gt; Parser(a, b, c, d)\n Defer the creation of a parser until it is needed. This is often most useful\n when creating a parser that is recursive and is *not* a function.\n\nSynonyms:\nchomp.lazy\nchomp lazy","url":"chomp.html#lazy"},{"doc":"chomp","title":"loop","content":"pub fn loop(\n  init: a,\n  step: fn(a) -&gt; Parser(Loop(b, a), c, d, e),\n) -&gt; Parser(b, c, d, e)\n\n\nSynonyms:\nchomp.loop\nchomp loop","url":"chomp.html#loop"},{"doc":"chomp","title":"many","content":"pub fn many(\n  parser: Parser(a, b, c, d),\n) -&gt; Parser(List(a), b, c, d)\n Parse zero or more of the given parser.\n\nSynonyms:\nchomp.many\nchomp many","url":"chomp.html#many"},{"doc":"chomp","title":"many1","content":"pub fn many1(\n  parser: Parser(a, b, c, d),\n) -&gt; Parser(List(a), b, c, d)\n Parse one or more of the given parser.\n\n 💡 If this parser succeeds, the list produced is guaranteed to be non-empty.\n Feel free to `let assert` the result!\n\nSynonyms:\nchomp.many1\nchomp many1","url":"chomp.html#many1"},{"doc":"chomp","title":"map","content":"pub fn map(\n  parser: Parser(a, b, c, d),\n  f: fn(a) -&gt; e,\n) -&gt; Parser(e, b, c, d)\n Run `parser`, applying `f` to its result.\n\nSynonyms:\nchomp.map\nchomp map","url":"chomp.html#map"},{"doc":"chomp","title":"map_error","content":"pub fn map_error(\n  parser: Parser(a, b, c, d),\n  map: fn(Error(b, c)) -&gt; Error(b, c),\n) -&gt; Parser(a, b, c, d)\n Run a parser and if it fails and did not consume any tokens, apply the given\n function to the error and return the new error. This function provides a\n way to add more information to error messages. For example, if you just parsed\n some indentation in an indentation-sensitive language and now want to parse\n a statement, you could map the statement parser&#39;s error to add extra info:\n\n \n fn parse_statement() {\n   // parse a statement\n   |&gt; chomp.or_error(&quot;I wanted a statement&quot;)\n }\n\n fn parse_something() {\n   use _ &lt;- do(parse_indentation())\n   use statement &lt;- do(\n     parse_statement() |&gt; chomp.map_error(because_of_indent)\n   )\n   // ...\n }\n\n fn because_of_indent(error) {\n   case error {\n     chomp.Custom(error) -&gt;\n       chomp.Custom(&quot;Since there was an indent, &quot; &lt;&gt; error)\n     _ -&gt; error\n   }\n }\n \n\n Now if parsing the statement in `parse_something` fails, the error message\n will be `Since there was an indent, I wanted a statement`. Sweet!\n\n Note that, like `or_error`, `EndOfInput` errors are not changed to minimize\n confusing error messages.\n\nSynonyms:\nchomp.map_error\nchomp map_error","url":"chomp.html#map_error"},{"doc":"chomp","title":"one_of","content":"pub fn one_of(\n  parsers: List(Parser(a, b, c, d)),\n) -&gt; Parser(a, b, c, d)\n Try each parser in order until one succeeds. If none succeed, the last parser&#39;s\n error is returned. It is recommended to use a custom error with\n [`or_error`](#or_error) for better error messages.\n\nSynonyms:\nchomp.one_of\nchomp one_of","url":"chomp.html#one_of"},{"doc":"chomp","title":"optional","content":"pub fn optional(\n  parser: Parser(a, b, c, d),\n) -&gt; Parser(Option(a), b, c, d)\n Try the given parser, but if it fails return\n [`None`](#https://hexdocs.pm/gleam_stdlib/gleam/option.html#Option) instead\n of failing.\n\nSynonyms:\nchomp.optional\nchomp optional","url":"chomp.html#optional"},{"doc":"chomp","title":"or","content":"pub fn or(\n  parser: Parser(a, b, c, d),\n  default: a,\n) -&gt; Parser(a, b, c, d)\n Try the given parser, but if it fails return the given default value instead\n of failing.\n\nSynonyms:\nchomp.or\nchomp or","url":"chomp.html#or"},{"doc":"chomp","title":"or_error","content":"pub fn or_error(\n  parser: Parser(a, b, c, d),\n  error: b,\n) -&gt; Parser(a, b, c, d)\n Run a parser and if it fails and did not consume any tokens, return the given\n error. This function is extremely useful for providing custom error messages\n to parsers such as `one_of`, `take_map`, and `take_if`.\n\n Note that `EndOfInput` errors are not replaced to minimize confusing error\n messages.\n\n \n import chomp\n\n fn value() {\n   chomp.one_of([\n     string(),\n     number(),\n     variable(),\n     list(),\n     group(),\n   ])\n   |&gt; chomp.or_error(&quot;I expected a value (string, number, variable, or list)&quot;)\n }\n \n\nSynonyms:\nchomp.or_error\nchomp or_error","url":"chomp.html#or_error"},{"doc":"chomp","title":"replace","content":"pub fn replace(\n  parser: Parser(a, b, c, d),\n  with b: e,\n) -&gt; Parser(e, b, c, d)\n Run `parser`, replacing its result with `b`.\n\nSynonyms:\nchomp.replace\nchomp replace","url":"chomp.html#replace"},{"doc":"chomp","title":"replace_error","content":"pub fn replace_error(\n  parser: Parser(a, b, c, d),\n  error: b,\n) -&gt; Parser(a, b, c, d)\n Run a parser and if it fails, return the given error. This parser is similar\n to [`or_error`](#or_error), but it&#39;s not quite the same—this one does not\n care whether the parser consumed any tokens when it failed. That means that\n if you have a parser like this:\n\n \n chomp.one_of([function(), constant()])\n |&gt; chomp.replace_error(&quot;I expected a function or constant&quot;)\n \n\n You will always get the error message &quot;I expected a function or constant&quot; even\n if any of the `one_of` parsers consumed tokens—a behavior that probably isn&#39;t\n what you want.\n\n For instance, if this was parsed:\n\n js\n function 23() {}\n //       ^^ this is the error\n \n\n We would get the error message &quot;I expected a function or constant&quot; because the\n `function` parser&#39;s error was swallowed by `replace_error`. Contrast that to\n `or_error`, where we would get something much better such as &quot;I expected an\n identifier&quot;.\n\n Of course there *are* cases where `replace_error` can be helpful; just be\n aware that you&#39;ll often want to reach for `or_error` first.\n\nSynonyms:\nchomp.replace_error\nchomp replace_error","url":"chomp.html#replace_error"},{"doc":"chomp","title":"return","content":"pub fn return(value: a) -&gt; Parser(a, b, c, d)\n The simplest kind of parser. [`return`](#return) consumes no tokens and always\n produces the given value. Sometimes called [`succeed`](#succeed) instead.\n\n This function might seem useless at first, but it is very useful when used in\n combination with [`do`](#do) or [`then`](#then).\n\n \n import chomp.{do, return}\n\n fn unit8_parser() {\n   use int &lt;- do(int_parser())\n\n   case int &gt;= 0, int &lt;= 255 {\n     True, True -&gt;\n       return(int)\n\n     False, _ -&gt;\n       throw(&quot;Expected an int &gt;= 0&quot;)\n\n     _, False -&gt;\n       throw(&quot;Expected an int &lt;= 255&quot;)\n  }\n }\n \n\n 💡 [`return`](#return) and [`succeed`](#succeed) are names for the same thing.\n We suggesting using `return` unqualified when using `do` and Gleam&#39;s `use`\n syntax, and `chomp.succeed` in a pipeline with `chomp.then`.\n\nSynonyms:\nchomp.return\nchomp return","url":"chomp.html#return"},{"doc":"chomp","title":"run","content":"pub fn run(\n  src: List(Token(a)),\n  parser: Parser(b, c, a, d),\n) -&gt; Result(b, #(Error(c, a), Span, List(#(Span, d))))\n Parsers don&#39;t do anything until they&#39;re run! The `run` function takes a\n [`Parser`](#Parser) and a list of [`Token`](./chomp/lexer#Token)s and\n runs it; returning either the parsed value or a tuple of the [`Error`](#Error)\n with the position where the parser failed and the final context stack.\n\nSynonyms:\nchomp.run\nchomp run","url":"chomp.html#run"},{"doc":"chomp","title":"sequence","content":"pub fn sequence(\n  parser: Parser(a, b, c, d),\n  separator sep: Parser(e, b, c, d),\n) -&gt; Parser(List(a), b, c, d)\n Parse a sequence separated by the `separator` parser.\n\nSynonyms:\nchomp.sequence\nchomp sequence","url":"chomp.html#sequence"},{"doc":"chomp","title":"succeed","content":"pub fn succeed(value: a) -&gt; Parser(a, b, c, d)\n💡 [`succeed`](#succeed) and [`return`](#return) are names for the same thing.\n We suggest using `succeed` in a pipeline with `chomp.then`, and `return`\n unqalified when using `do` with Gleam&#39;s `use` syntax.\n\nSynonyms:\nchomp.succeed\nchomp succeed","url":"chomp.html#succeed"},{"doc":"chomp","title":"take_if","content":"pub fn take_if(predicate: fn(a) -&gt; Bool) -&gt; Parser(a, b, a, c)\n Take and return a token if it satisfies the predicate.\n\nSynonyms:\nchomp.take_if\nchomp take_if","url":"chomp.html#take_if"},{"doc":"chomp","title":"take_map","content":"pub fn take_map(f: fn(a) -&gt; Option(b)) -&gt; Parser(b, c, a, d)\n Take the next token and attempt to transform it with the given function. This\n is useful when creating reusable primtive parsers for your own tokens such as\n `take_identifier` or `take_number`.\n\nSynonyms:\nchomp.take_map\nchomp take_map","url":"chomp.html#take_map"},{"doc":"chomp","title":"then","content":"pub fn then(\n  parser: Parser(a, b, c, d),\n  f: fn(a) -&gt; Parser(e, b, c, d),\n) -&gt; Parser(e, b, c, d)\n Another name for [`do`](#do).\n\nSynonyms:\nchomp.then\nchomp then","url":"chomp.html#then"},{"doc":"chomp","title":"throw","content":"pub fn throw(error: a) -&gt; Parser(b, a, c, d)\n The opposite of [`return`](#return), this parser always fails with the given\n error. Sometimes called [`fail`](#fail) instead.\n\nSynonyms:\nchomp.throw\nchomp throw","url":"chomp.html#throw"},{"doc":"chomp","title":"token","content":"pub fn token(tok: a) -&gt; Parser(Span, b, a, c)\n Parse a token of a particular type, returning its position.\n\n \n use start_pos &lt;- do(chomp.token(LParen))\n // ...\n use end_pos &lt;- do(chomp.token(RParen))\n \n\nSynonyms:\nchomp.token\nchomp token","url":"chomp.html#token"},{"doc":"chomp","title":"until","content":"pub fn until(\n  parser: Parser(a, b, c, d),\n  tok: c,\n) -&gt; Parser(List(a), b, c, d)\n Parse until the given token is encountered, returning a list of the results.\n\nSynonyms:\nchomp.until\nchomp until","url":"chomp.html#until"},{"doc":"chomp","title":"until_end","content":"pub fn until_end(\n  parser: Parser(a, b, c, d),\n) -&gt; Parser(List(a), b, c, d)\n Parse until the end of the token stream, returning a list of the results.\n\n 💡 This parser produces better error messages than using both [`many`](#many)\n and [`end`](#end) in order, so you&#39;ll often want to use this function instead.\n A common use-case is when you&#39;re parsing top-level statements or expressions\n in a programming language until the end of the file.\n\n \n use statements &lt;- do(chomp.until_end(statement()))\n return(statements)\n\n // is generally better than\n\n use statements &lt;- do(chomp.many(statement()))\n use _ &lt;- do(chomp.end())\n return(statements)\n \n\nSynonyms:\nchomp.until_end\nchomp until_end","url":"chomp.html#until_end"},{"doc":"chomp","title":"chomp","content":"","url":"chomp.html"},{"doc":"chomp/pratt","title":"Config","content":"pub opaque type Config(a, e, tok, ctx)\n\n\nSynonyms:\nchomp/pratt.Config\nchomp/pratt Config","url":"chomp/pratt.html#Config"},{"doc":"chomp/pratt","title":"Operator","content":"pub opaque type Operator(a, e, tok, ctx)\n\n\nSynonyms:\nchomp/pratt.Operator\nchomp/pratt Operator","url":"chomp/pratt.html#Operator"},{"doc":"chomp/pratt","title":"Precedence","content":"pub type Precedence {\n  Left(Int)\n  Right(Int)\n}\n\nLeft(Int)\n\n\nRight(Int)\n\n\nSynonyms:\nchomp/pratt.Precedence\nchomp/pratt Precedence","url":"chomp/pratt.html#Precedence"},{"doc":"chomp/pratt","title":"expression","content":"pub fn expression(\n  one_of first: List(fn(Config(a, b, c, d)) -&gt; Parser(a, b, c, d)),\n  or_error or_error: b,\n  and_then then: List(Operator(a, b, c, d)),\n) -&gt; Parser(a, b, c, d)\n\nSynonyms:\nchomp/pratt.expression\nchomp/pratt expression","url":"chomp/pratt.html#expression"},{"doc":"chomp/pratt","title":"infix","content":"pub fn infix(\n  precedence: Precedence,\n  operator: Parser(a, b, c, d),\n  apply: fn(e, e) -&gt; e,\n) -&gt; Operator(e, b, c, d)\n\nSynonyms:\nchomp/pratt.infix\nchomp/pratt infix","url":"chomp/pratt.html#infix"},{"doc":"chomp/pratt","title":"infix_custom","content":"pub fn infix_custom(\n  precedence: Precedence,\n  operator: fn(Config(a, b, c, d)) -&gt; Parser(e, b, c, d),\n  apply: fn(a, a, e) -&gt; a,\n) -&gt; Operator(a, b, c, d)\n\nSynonyms:\nchomp/pratt.infix_custom\nchomp/pratt infix_custom","url":"chomp/pratt.html#infix_custom"},{"doc":"chomp/pratt","title":"operator_custom","content":"pub fn operator_custom(\n  build: fn(Config(a, b, c, d)) -&gt;\n    #(Int, fn(a) -&gt; Parser(a, b, c, d)),\n) -&gt; Operator(a, b, c, d)\n\nSynonyms:\nchomp/pratt.operator_custom\nchomp/pratt operator_custom","url":"chomp/pratt.html#operator_custom"},{"doc":"chomp/pratt","title":"postfix","content":"pub fn postfix(\n  precedence: Int,\n  operator: Parser(a, b, c, d),\n  apply: fn(e) -&gt; e,\n) -&gt; Operator(e, b, c, d)\n\nSynonyms:\nchomp/pratt.postfix\nchomp/pratt postfix","url":"chomp/pratt.html#postfix"},{"doc":"chomp/pratt","title":"postfix_custom","content":"pub fn postfix_custom(\n  precedence: Int,\n  operator: fn(Config(a, b, c, d)) -&gt; Parser(e, b, c, d),\n  apply: fn(a, e) -&gt; a,\n) -&gt; Operator(a, b, c, d)\n\nSynonyms:\nchomp/pratt.postfix_custom\nchomp/pratt postfix_custom","url":"chomp/pratt.html#postfix_custom"},{"doc":"chomp/pratt","title":"prefix","content":"pub fn prefix(\n  precedence: Int,\n  operator: Parser(a, b, c, d),\n  apply: fn(e) -&gt; e,\n) -&gt; fn(Config(e, b, c, d)) -&gt; Parser(e, b, c, d)\n\nSynonyms:\nchomp/pratt.prefix\nchomp/pratt prefix","url":"chomp/pratt.html#prefix"},{"doc":"chomp/pratt","title":"prefix_custom","content":"pub fn prefix_custom(\n  precedence: Int,\n  operator: fn(Config(a, b, c, d)) -&gt; Parser(e, b, c, d),\n  apply: fn(a, e) -&gt; a,\n) -&gt; fn(Config(a, b, c, d)) -&gt; Parser(a, b, c, d)\n\nSynonyms:\nchomp/pratt.prefix_custom\nchomp/pratt prefix_custom","url":"chomp/pratt.html#prefix_custom"},{"doc":"chomp/pratt","title":"sub_expression","content":"pub fn sub_expression(\n  config: Config(a, b, c, d),\n  precedence: Int,\n) -&gt; Parser(a, b, c, d)\n\nSynonyms:\nchomp/pratt.sub_expression\nchomp/pratt sub_expression","url":"chomp/pratt.html#sub_expression"},{"doc":"chomp/pratt","title":"chomp/pratt","content":" This module is very similar to the\n [`elm-pratt-parser`](https://package.elm-lang.org/packages/dmy/elm-pratt-parser/latest/Pratt)\n package in the Elm ecosystem, and much of its docs apply to this module.","url":"chomp/pratt.html"}]);